/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#include <cmath>

#include "LEDParamController.h"
#include "controllers/common/family/Families.h"

using namespace std;

namespace stereo_daq {

const string LEDParamController::TYPE = "ledparam_controller";


/*
 * Constructor
 */
LEDParamController::LEDParamController(const string& name) :
	ExperimentController(name),controller::Stoppable(this),Select(this),Set(this) {

	setFamily(family::ACQUISITION,family::SETTING);

	// Init property
	delay.init(this, SAVE, "delay");
	period.init(this, SAVE, "period");
	box.init(this, NOSAVE, "box");
	pattern.init(this, NOSAVE, "pattern");
	nbled.init(this, NOSAVE, "nbled");
	dacbox0.init(this, NOSAVE, "dacbox0");
	dacbox1.init(this, NOSAVE, "dacbox1");
	dacbox2.init(this, NOSAVE, "dacbox2");
	dacbox3.init(this, NOSAVE, "dacbox3");
	dacbox4.init(this, NOSAVE, "dacbox4");

	m_ELECDriver.init(this, "driver");

	m_StartActivated = false;

	registerFunction(TYPE);

}

LEDParamController::LEDParamController(const LEDParamController& controller) :
	ExperimentController(controller),controller::Stoppable(this), Select(this),Set(this) {

	delay.copy(this, controller.delay);
	period.copy(this, controller.period);
	box.copy(this, controller.box);
	pattern.copy(this, controller.pattern);
	nbled.copy(this, controller.nbled);
	dacbox0.copy(this, controller.dacbox0);
	dacbox1.copy(this, controller.dacbox1);
	dacbox2.copy(this, controller.dacbox2);
	dacbox3.copy(this, controller.dacbox3);
	dacbox4.copy(this, controller.dacbox4);
	m_ELECDriver.copy(controller.m_ELECDriver);

	m_StartActivated = false;

}

/*
 * Destructor
 */
LEDParamController::~LEDParamController() {
	// Empty
}

void LEDParamController::postConfiguration() {

	nbled=stereo_elec::NBLEDPERBOARD;
	dacbox0.resize(nbled());
	dacbox1.resize(nbled());
	dacbox2.resize(nbled());
	dacbox3.resize(nbled());
	dacbox4.resize(nbled());

	registerRefresher(delay, &LEDParamController::refreshDelayProperty, this);
	registerRefresher(period, &LEDParamController::refreshPeriodProperty, this);
	registerRefresher(box, &LEDParamController::refreshBoxProperty, this);
	registerRefresher(pattern, &LEDParamController::refreshPatternProperty, this);
	registerRefresher(dacbox0, &LEDParamController::refreshDACBox0Property, this);
	registerRefresher(dacbox1, &LEDParamController::refreshDACBox1Property, this);
	registerRefresher(dacbox2, &LEDParamController::refreshDACBox2Property, this);
	registerRefresher(dacbox3, &LEDParamController::refreshDACBox3Property, this);
	registerRefresher(dacbox4, &LEDParamController::refreshDACBox4Property, this);

	registerProgression(m_ELECDriver, &LEDParamController::updateProgression, this);


}

void LEDParamController::refreshDelayProperty(int32 value) throw (CannotSetValue) {
	m_ELECDriver->DAQParams.ledPulseDlyVal = (short unsigned) value;
}

void LEDParamController::refreshPeriodProperty(int32 value) throw (CannotSetValue) {
	m_ELECDriver->DAQParams.ledPeriod = (short unsigned) value;
}
void LEDParamController::refreshBoxProperty(int32 value) throw (CannotSetValue) {
	m_ELECDriver->DAQParams.ledBox = (short unsigned) value;
}
void LEDParamController::refreshPatternProperty(int32 value) throw (CannotSetValue) {
	m_ELECDriver->DAQParams.ledType = (short unsigned) value;
}
void LEDParamController::refreshDACBox0Property(int32 index,int32 value) throw (CannotSetValue) {
	m_ELECDriver->DAQParams.ledDAC.at(0*nbled()+index) = (short unsigned) value;
}
void LEDParamController::refreshDACBox1Property(int32 index,int32 value) throw (CannotSetValue) {
	m_ELECDriver->DAQParams.ledDAC[1*nbled()+index] = (short unsigned) value;
}
void LEDParamController::refreshDACBox2Property(int32 index,int32 value) throw (CannotSetValue) {
	m_ELECDriver->DAQParams.ledDAC[2*nbled()+index] = (short unsigned) value;
}
void LEDParamController::refreshDACBox3Property(int32 index,int32 value) throw (CannotSetValue) {
	m_ELECDriver->DAQParams.ledDAC[3*nbled()+index] = (short unsigned) value;
}
void LEDParamController::refreshDACBox4Property(int32 index,int32 value) throw (CannotSetValue) {
	m_ELECDriver->DAQParams.ledDAC[4*nbled()+index] = (short unsigned) value;
}

/*
 * updateProgression
 */
void LEDParamController::updateProgression() {
	// Calculate its own progression to manage retry case
	int32 progression = m_ELECDriver->commandProgression();
	if (progression >= 100) {
		progression = 99;
	}
	commandProgression = progression;
}

/*
 * select channels
 */
void LEDParamController::select() {


}

void LEDParamController::set() {

}
void LEDParamController::start() {


}

void LEDParamController::stop() {

}

}

