/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#include "RealUsbPVCamDriver.h"
#include "UsbPVCamDriver.h"

#include <cstdio>

namespace usbpvcam {

/*
 * Constructor
 */
RealUsbPVCamDriver::RealUsbPVCamDriver(UsbPVCamDriver* owner) :
		UsbPVCamState(owner) {
	/* Empty */
}

/*
 * Destructor
 */
RealUsbPVCamDriver::~RealUsbPVCamDriver() {
	plExpUninitSeq();
	plCamClose(m_Handle);
	plPVCamUninit();

}

/*
 * init
 */
void RealUsbPVCamDriver::init() {
	if (!plPVCamInit()) {
		cerr << "failed to init pvcam" << endl;
		print_pv_error();
	} else {
		char cam_name[CAM_NAME_LEN];
		if (plCamGetName(0, cam_name)) {
			cerr << "camname for cam 0 is " << cam_name << endl;
			if (plCamOpen(cam_name, &m_Handle)) {
			} else {
				cerr << "camera " << cam_name << " didn't open" << endl;
				print_pv_error();
			}
		} else {
			cerr << "didn't get cam name" << endl;
			print_pv_error();
		}
	}
	readInfos();
	readStatus();
}

/*
 * clear
 */
void RealUsbPVCamDriver::clear() {
	memset(owner()->data(), 0, owner()->dataSize() * sizeof(int32));
}

/*
 * writeParam
 */
void RealUsbPVCamDriver::writeParam() {
}

/*
 * synchroniseRead
 */
void RealUsbPVCamDriver::synchroniseRead() {

	uint16* datas = owner()->m_DataShort;
	/*
	 * Calculate the detector sum here
	 */
	int32* data = owner()->data();
	int32* mask = owner()->mask();
	int32 dataSize = owner()->dataSize();
	float64 sum = 0;
	if (datas != NULL) {
		for (int32 i = 0; i < dataSize; ++i) {
			data[mask[i]] = datas[i];
			//data[i] = datas[i];
			sum += (float64) data[i];
		}
	} else {
		cerr << "error mask " << endl;

	}
	owner()->sum = sum;
	owner()->data.sendEvent();

	/* Finish the sequence */
	plExpFinishSeq(m_Handle, (void*) datas, 0);

	/*Uninit the sequence */
//plExpUninitSeq();
	owner()->m_SynchroniseReadTerminated = true;
}

/*
 * read
 */
void RealUsbPVCamDriver::read() {
}

/*
 * start
 */
void RealUsbPVCamDriver::start() {

	uint32 exp_time = (uint32) (owner()->time.setpoint() * owner()->precision());
clear();
	unsigned long size;
	void *region = allocRegion();

	if (region) {
		uint16 param;

		setS1(region, 0);
		plGetParam(m_Handle, PARAM_SER_SIZE, pvcam::ATTR_DEFAULT, (void *) &param);
		setS2(region, 1023);
		setSBin(region, 1);
		setP1(region, 0);
		plGetParam(m_Handle, PARAM_PAR_SIZE, pvcam::ATTR_DEFAULT, (void *) &param);
		setP2(region, 1023);
		setPBin(region, 1);

		if (plExpInitSeq()) {
			if (plExpSetupSeq(m_Handle, 1, 1, region, pvcam::TIMED_MODE, exp_time, &size)) {
				uint16* datas = owner()->m_DataShort;
				if (plExpStartSeq(m_Handle, (void*) datas)) {
					m_TimeCounter.clear();
				} else {
					print_pv_error();
				}

			} else {
				cerr << "experiment setup failed" << endl;
			}
		} else {
			cerr << "init_seq failed" << endl;
		}
		free(region);
	}
}

/*
 * resume
 */
void RealUsbPVCamDriver::resume() {
}

/*
 * pause
 */
void RealUsbPVCamDriver::pause() {
}

/*
 * stop
 */
void RealUsbPVCamDriver::stop() {
	plExpAbort(m_Handle, pvcam::CCS_HALT_CLOSE_SHTR);
}

/*
 * readStatus
 */
void RealUsbPVCamDriver::readStatus() {

	/*
	 * Read the status card
	 */
	int16 value;
	unsigned long not_needed;
	plExpCheckStatus(m_Handle, &value, &not_needed);

	// Status command
	owner()->computeGateStatus(value);
	owner()->computeAcquisitionStatus(0);
	owner()->computeDetectorStatus(0);

	if (owner()->startActivated() == true) {
		if ((value == pvcam::READOUT_COMPLETE) || (value == pvcam::READOUT_FAILED)) {
			if (value == pvcam::READOUT_FAILED) {
				cerr << "Data collection error: " << plErrorCode() << endl;
			}
			plExpAbort(m_Handle, pvcam::CCS_HALT_CLOSE_SHTR);
			owner()->startActivated = false;
			synchroniseRead();
			owner()->time = owner()->time.setpoint();
			owner()->commandProgression = PROGRESSION_END_DEVICE_CONTAINER;
		} else {
			float64 aTime = m_TimeCounter.getTime();
			if (aTime <= owner()->time.setpoint()) {
				owner()->time = aTime;
			}
			owner()->calculateProgression();
		}
	} else {
		if (owner()->m_SynchroniseReadActivated == true) {
			if (owner()->m_SynchroniseReadTerminated == true) {
				owner()->m_SynchroniseReadActivated = false;
				owner()->m_SynchroniseReadTerminated = false;
				owner()->commandProgression = PROGRESSION_END_DEVICE_CONTAINER;
			}
		} else {
			owner()->commandProgression = PROGRESSION_END_DEVICE_CONTAINER;
		}
	}
//	}

}

/*
 * readInfos
 */
void RealUsbPVCamDriver::readInfos() {
}

/*
 * print_pv_error
 */
void RealUsbPVCamDriver::print_pv_error() {
	char msg[ERROR_MSG_LEN];
	int16 i;

	i = plErrorCode();
	plErrorMessage(i, msg);
	printf(" [%i]%s\n", i, msg);
}

}
